// Copyright 2015-2021 Espressif Systems (Shanghai) CO LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "hmc8073.h"

typedef struct {
    spi_device_handle_t spi_dev;
    uint8_t address;
} hmc8073_dev_t;

esp_err_t hmc8073_set(hmc8073_handle_t att, uint8_t att_val)
{
    if (att_val > MAX_ATT_SUPPORTED) {
        return ESP_ERR_INVALID_ARG;
    }

    hmc8073_dev_t *att_dev = (hmc8073_dev_t *) att;

    spi_transaction_t t = {.length = 16,
                           .tx_data = {att_val << 1, att_dev->address},
                           .flags = SPI_TRANS_USE_TXDATA
                          };

    return spi_device_queue_trans(att_dev->spi_dev, &t, 1000);
}

hmc8073_handle_t hmc8073_create(spi_host_device_t spi_host, uint8_t addr, int cs_pin)
{
    hmc8073_dev_t *attenuator = (hmc8073_dev_t *)calloc(1, sizeof(hmc8073_dev_t));
    if (attenuator != NULL) {

        const spi_device_interface_config_t devcfg = {
            .command_bits = 0,
            .address_bits = 0,
            .clock_speed_hz = 1 * 1000 * 1000, // Clock out at 1 MHz
            .mode = 0,                         // SPI mode 0
            .spics_io_num = cs_pin,
            .queue_size = 4,
            .flags = SPI_DEVICE_TXBIT_LSBFIRST // Transmit LSB first
        };

        ESP_ERROR_CHECK(spi_bus_add_device(spi_host, &devcfg, &attenuator->spi_dev));
        attenuator->address = 0x07 & addr;
    }

    return (hmc8073_handle_t)attenuator;
}

void hmc8073_delete(hmc8073_handle_t att)
{
    hmc8073_dev_t *att_dev = (hmc8073_dev_t *) att;
    spi_bus_remove_device(att_dev->spi_dev);
    free(att);
}
