// Copyright 2015-2021 Espressif Systems (Shanghai) CO LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

#include "driver/spi_master.h"
#define MAX_ATT_SUPPORTED       63u /*!< Attenuation range: 0 - 31.5dB in 0.5dB steps */

#ifdef __cplusplus
extern "C" {
#endif

typedef void *hmc8073_handle_t;

/**
 * @brief Create HMC8073 attenuator object and return its handle
 *
 * @attention User is responsible for spi_host configuration with `spi_bus_initialize`.
 *
 * @param spi_host
 * @param addr
 * @param cs_pin
 * @return hmc8073_handle_t
 */
hmc8073_handle_t hmc8073_create(spi_host_device_t spi_host, const uint8_t addr, const int cs_pin);

/**
 * @brief Set attenuation
 *
 * @param att HMC8073 handle
 * @param att_val Attenuation. 0-31.5dB in 0.5dB steps
 * @return
 *     - ESP_OK Success
 *     - ESP_ERR_INVALID_ARG Argument out of range
 */
esp_err_t hmc8073_set(hmc8073_handle_t att, const uint8_t att_val);

/**
 * @brief Delete and release a sensor object
 *
 * @param att HMC8073 handle
 */
void hmc8073_delete(hmc8073_handle_t att);

#ifdef __cplusplus
}
#endif
